@@ -2,7 +2,7 @@
 
 #
 # KLayout Layout Viewer
-# Copyright (C) 2006-2018 Matthias Koefferlein
+# Copyright (C) 2006-2025 Matthias Koefferlein
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,11 +22,25 @@
 CURR_DIR=`pwd`
 RUN_MAKE=1
 IS_MAC="no"
+IS_WINDOWS="no"
+IS_LINUX="no"
 
 HAVE_QTBINDINGS=1
+HAVE_QT_UITOOLS=1
+HAVE_QT_NETWORK=1
+HAVE_QT_SQL=1
+HAVE_QT_SVG=1
+HAVE_QT_PRINTSUPPORT=1
+HAVE_QT_MULTIMEDIA=1
+HAVE_QT_DESIGNER=1
+HAVE_QT_XML=1
 HAVE_64BIT_COORD=0
-HAVE_QT5=""
-HAVE_CURL=""
+HAVE_QT=1
+HAVE_PNG=0
+HAVE_CURL=0
+HAVE_EXPAT=0
+HAVE_GIT2=1
+HAVE_LSTREAM=1
 
 RUBYINCLUDE=""
 RUBYINCLUDE2=""
@@ -35,6 +49,7 @@ RUBYVERSIONCODE=""
 
 PYTHONINCLUDE=""
 PYTHONLIBFILE=""
+PYTHONEXTSUFFIX=""
 
 QMAKE=""
 RUBY=""
@@ -50,11 +65,19 @@ BUILD_EXPERT=0
 
 # Check if building on Mac OSX Darwin family
 case `uname` in
+    Linux*)
+        IS_LINUX="yes"
+        ;;
+    MINGW*)
+        IS_WINDOWS="yes"
+        ;;
+    CYGWIN*)
+        IS_WINDOWS="yes"
+        ;;
     Darwin*)
         IS_MAC="yes"
         ;;
     *)
-        IS_MAC="no"
         ;;
 esac
 
@@ -70,12 +93,39 @@ while [ "$*" != "" ]; do
   shift
 
   case $a in
+  -without-qt)
+    HAVE_QT=0
+    ;;
   -with-qtbinding)
     HAVE_QTBINDINGS=1
     ;;
   -without-qtbinding)
     HAVE_QTBINDINGS=0
     ;;
+  -without-qt-uitools)
+    HAVE_QT_UITOOLS=0
+    ;;
+  -without-qt-network)
+    HAVE_QT_NETWORK=0
+    ;;
+  -without-qt-sql)
+    HAVE_QT_SQL=0
+    ;;
+  -without-qt-svg)
+    HAVE_QT_SVG=0
+    ;;
+  -without-qt-printsupport)
+    HAVE_QT_PRINTSUPPORT=0
+    ;;
+  -without-qt-multimedia)
+    HAVE_QT_MULTIMEDIA=0
+    ;;
+  -without-qt-designer)
+    HAVE_QT_DESIGNER=0
+    ;;
+  -without-qt-xml)
+    HAVE_QT_XML=0
+    ;;
   -with-64bit-coord)
     HAVE_64BIT_COORD=1
     ;;
@@ -106,6 +156,10 @@ while [ "$*" != "" ]; do
     PYTHONLIBFILE="$1"
     shift
     ;;
+  -pyextsuffix)
+    PYTHONEXTSUFFIX="$1"
+    shift
+    ;;
   -qmake)
     QMAKE="$1"
     shift
@@ -145,18 +199,30 @@ while [ "$*" != "" ]; do
     RPATH="$1"
     shift
     ;;
-  -qt4)
-    HAVE_QT5=0
-    ;;
-  -qt5)
-    HAVE_QT5=1
-    ;;
   -dry-run)
     RUN_MAKE=0
     ;;
+  -libpng)
+    HAVE_PNG=1
+    ;;
   -libcurl)
     HAVE_CURL=1
     ;;
+  -libexpat)
+    HAVE_EXPAT=1
+    ;;
+  -nolibgit2)
+    HAVE_GIT2=0
+    ;;
+  -nolstream)
+    HAVE_LSTREAM=0
+    ;;
+  -qt5)
+    echo "*** WARNING: -qt5 option is ignored - Qt version is auto-detected now."
+    ;;
+  -qt4)
+    echo "*** WARNING: -qt4 option is ignored - Qt version is auto-detected now."
+    ;;
   -option)
     MAKE_OPT="$MAKE_OPT $1"
     shift
@@ -184,26 +250,35 @@ while [ "$*" != "" ]; do
     echo ""
     echo "  -with-qtbinding       Create Qt bindings for ruby scripts [default]"
     echo "  -without-qtbinding    Don't create Qt bindings for ruby scripts"
+    echo "  -without-qt-uitools   Don't include uitools in Qt binding"
     echo "  -with-64bit-coord     Use long (64bit) coordinates - EXPERIMENTAL FEATURE"
-    echo "                        (only available for gcc>=4.4 for 64bit build)"
+    echo "                          (only available for gcc>=4.4 for 64bit build)"
     echo "  -without-64bit-coord  Don't use long (64bit) coordinates [default]"
+    echo "  -without-qt           Qt-less build of the core libraries (including pymod)"
+    echo "                          (implies -without-qtbinding)"
     echo ""
     echo "  -dry-run              Don't build, just run qmake"
     echo ""
     echo "Special options (usually auto-selected from ruby/python/Qt installation):"
     echo ""
-    echo "  -qt4|-qt5             Use Qt4 or Qt5 API [default: auto detect]"
-    echo ""
     echo "  -rblib <file>         Location of the .so/.dll to link for Ruby support"
     echo "  -rbinc <dir>          Location of the Ruby headers (in particular 'ruby.h')"
-    echo "                        -rbinc and -rblib must be set to enable Ruby support"
+    echo "                          -rbinc and -rblib must be set to enable Ruby support"
     echo "  -rbinc2 <dir>         Second include path for Ruby 1.9 (containing 'ruby/config.h')"
     echo "  -rbvers <xyyzz>       Ruby version code"
     echo ""
     echo "  -pylib <file>         Location of the .so/.dll to link for Python support"
     echo "  -pyinc <dir>          Location of the Python headers (in particular 'Python.h')"
     echo ""
-    echo "  -libcurl              Use libcurl instead of QNetwork (for Qt<4.7)"
+    echo "  -libcurl              Use libcurl instead of QtNetwork (for Qt<4.7)"
+    echo "  -libexpat             Use libexpat instead of QtXml"
+    echo "  -libpng               Use libpng instead of Qt for PNG generation"
+    echo "  -nolibgit2            Do not include libgit2 for Git package support"
+    echo "  -nolstream            Do not include the LStream plugin"
+    echo ""
+    echo "Environment Variables:"
+    echo ""
+    echo "  QMAKE_CCACHE=1        Adds CONFIG+=ccache to qmake command (only works if Qt>=5.9.2)"
     echo ""
     exit 0
     ;;
@@ -239,25 +314,7 @@ if [ "$QMAKE" = "" ]; then
   exit 1
 fi
 
-# if not given, try to detect the qt major version to use
-if [ "$HAVE_QT5" = "" ]; then
-  qt_major=`$QMAKE -v | grep 'Using Qt version' | sed 's/.*version  *\([0-9][0-9]*\).*/\1/'`
-  if [ "$qt_major" = "4" ]; then
-    HAVE_QT5=0
-  elif [ "$qt_major" = "5" ]; then
-    HAVE_QT5=1
-  else
-    echo "*** ERROR: could not determine Qt version from '$QMAKE -v'"
-    exit 1
-  fi
-fi
-
 echo "Using qmake: $QMAKE"
-if [ "$HAVE_QT5" != "0" ]; then
-  echo "    Using Qt 5 API"
-else
-  echo "    Using Qt 4 API"
-fi
 echo ""
 
 # if not given, locate ruby interpreter (prefer 1.9, then default, finally 1.8 as fallback)
@@ -404,6 +461,20 @@ if [ "$PYTHON" != "" ] && [ "$PYTHON" != "-" ]; then
     echo "    Python headers found: $PYTHONINCLUDE"
   fi
 
+  if [ "$PYTHONEXTSUFFIX" = "" ]; then
+    PYTHONEXTSUFFIX=$($PYTHON -c "import sysconfig; print(sysconfig.get_config_var('EXT_SUFFIX') or sysconfig.get_config_var('SO'))" 2>/dev/null)
+    if [ "$PYTHONEXTSUFFIX" = "" ]; then
+      if [ "$IS_WINDOWS" = "yes" ]; then
+        PYTHONEXTSUFFIX=.dll
+      elif [ "$IS_MAC" = "yes" ]; then
+        PYTHONEXTSUFFIX=.dylib
+      else
+        PYTHONEXTSUFFIX=.so
+      fi
+    fi
+    echo "    Python extension suffix: $PYTHONEXTSUFFIX"
+  fi
+
   echo "    Python installation is in:"
   echo "    - $PYTHONLIBFILE (lib)"
   echo "    - $PYTHONINCLUDE (includes)"
@@ -411,13 +482,35 @@ if [ "$PYTHON" != "" ] && [ "$PYTHON" != "-" ]; then
 
 fi
 
+if [ $HAVE_QT = 0 ]; then
+  HAVE_QTBINDINGS=0
+fi
+
 echo "Features:"
+if [ $HAVE_QT = 0 ]; then
+  echo "    Qt not used at all"
+fi
 if [ $HAVE_QTBINDINGS != 0 ]; then
   echo "    Qt bindings enabled"
 fi
 if [ $HAVE_64BIT_COORD != 0 ]; then
   echo "    64 bit coordinates enabled"
 fi
+if [ $HAVE_EXPAT != 0 ]; then
+  echo "    Uses libexpat for XML parsing"
+fi
+if [ $HAVE_CURL != 0 ]; then
+  echo "    Uses libcurl for network access"
+fi
+if [ $HAVE_PNG != 0 ]; then
+  echo "    Uses libpng for PNG generation"
+fi
+if [ $HAVE_GIT2 != 0 ]; then
+  echo "    Uses libgit2 for Git access"
+fi
+if [ $HAVE_LSTREAM != 0 ]; then
+  echo "    Includes LStream plugin"
+fi
 if [ "$RPATH" = "" ]; then
   RPATH="$BIN"
 fi
@@ -476,8 +569,34 @@ if [ "$BIN" = "" ]; then
   BIN=$CURR_DIR/bin-$CONFIG
 fi
 
+if [ "$QMAKE_CCACHE" = 1 ]; then
+  echo "    Compilation caching is activated."
+else
+  echo "    Compilation caching is deactivated!"
+fi
 echo "    Installation target: $BIN"
 echo "    Build directory: $BUILD"
+echo ""
+echo "    Build flags:"
+echo "      HAVE_RUBY=$HAVE_RUBY"
+echo "      HAVE_PYTHON=$HAVE_PYTHON"
+echo "      HAVE_QTBINDINGS=$HAVE_QTBINDINGS"
+echo "      HAVE_QT=$HAVE_QT"
+echo "      HAVE_QT_UITOOLS=$HAVE_QT_UITOOLS"
+echo "      HAVE_QT_NETWORK=$HAVE_QT_NETWORK"
+echo "      HAVE_QT_SQL=$HAVE_QT_SQL"
+echo "      HAVE_QT_SVG=$HAVE_QT_SVG"
+echo "      HAVE_QT_PRINTSUPPORT=$HAVE_QT_PRINTSUPPORT"
+echo "      HAVE_QT_MULTIMEDIA=$HAVE_QT_MULTIMEDIA"
+echo "      HAVE_QT_DESIGNER=$HAVE_QT_DESIGNER"
+echo "      HAVE_QT_XML=$HAVE_QT_XML"
+echo "      HAVE_64BIT_COORD=$HAVE_64BIT_COORD"
+echo "      HAVE_CURL=$HAVE_CURL"
+echo "      HAVE_PNG=$HAVE_PNG"
+echo "      HAVE_EXPAT=$HAVE_EXPAT"
+echo "      HAVE_GIT2=$HAVE_GIT2"
+echo "      HAVE_LSTREAM=$HAVE_LSTREAM"
+echo "      RPATH=$RPATH"
 
 mkdir -p $BUILD
 
@@ -519,13 +638,6 @@ echo ""
 echo "Running $QMAKE .."
 cd $BUILD
 
-# chose the right qmake
-if [ $HAVE_QT5 = 0 ]; then
-  export QT_SELECT=4
-else
-  export QT_SELECT=5
-fi
-
 $QMAKE -v
 
 # Force a minimum rebuild because of version info
@@ -535,17 +647,29 @@ qmake_options=(
   -recursive
   CONFIG+="$CONFIG"
   RUBYLIBFILE="$RUBYLIBFILE"
-  RUBYINCLUDE="$RUBYINCLUDE"
-  RUBYINCLUDE2="$RUBYINCLUDE2"
   RUBYVERSIONCODE="$RUBYVERSIONCODE"
   HAVE_RUBY="$HAVE_RUBY"
+  PYTHON="$PYTHON"
   PYTHONLIBFILE="$PYTHONLIBFILE"
   PYTHONINCLUDE="$PYTHONINCLUDE"
+  PYTHONEXTSUFFIX="$PYTHONEXTSUFFIX"
   HAVE_PYTHON="$HAVE_PYTHON"
   HAVE_QTBINDINGS="$HAVE_QTBINDINGS"
+  HAVE_QT_UITOOLS="$HAVE_QT_UITOOLS"
+  HAVE_QT_NETWORK="$HAVE_QT_NETWORK"
+  HAVE_QT_SQL="$HAVE_QT_SQL"
+  HAVE_QT_SVG="$HAVE_QT_SVG"
+  HAVE_QT_PRINTSUPPORT="$HAVE_QT_PRINTSUPPORT"
+  HAVE_QT_MULTIMEDIA="$HAVE_QT_MULTIMEDIA"
+  HAVE_QT_DESIGNER="$HAVE_QT_DESIGNER"
+  HAVE_QT_XML="$HAVE_QT_XML"
   HAVE_64BIT_COORD="$HAVE_64BIT_COORD"
-  HAVE_QT5="$HAVE_QT5"
+  HAVE_QT="$HAVE_QT"
   HAVE_CURL="$HAVE_CURL"
+  HAVE_EXPAT="$HAVE_EXPAT"
+  HAVE_PNG="$HAVE_PNG"
+  HAVE_GIT2="$HAVE_GIT2"
+  HAVE_LSTREAM="$HAVE_LSTREAM"
   PREFIX="$BIN"
   RPATH="$RPATH"
   KLAYOUT_VERSION="$KLAYOUT_VERSION"
@@ -553,6 +677,24 @@ qmake_options=(
   KLAYOUT_VERSION_REV="$KLAYOUT_VERSION_REV"
 )
 
+# NOTE: qmake does not like include paths which clash with paths built into the compiler
+# hence we don't add RUBYINCLUDE or RUBYINCLUDE2 in this case (found on CentOS 8 where Ruby
+# headers are installed in /usr/include)
+if [ "$RUBYINCLUDE" != "/usr/include" ] && [  "$RUBYINCLUDE" != "/usr/local/include" ]; then
+  qmake_options+=( RUBYINCLUDE="$RUBYINCLUDE" )
+fi
+if [ "$RUBYINCLUDE2" != "/usr/include" ] && [  "$RUBYINCLUDE2" != "/usr/local/include" ]; then
+  qmake_options+=( RUBYINCLUDE2="$RUBYINCLUDE2" )
+fi
+
+# This should speed up build time considerably
+# https://ortogonal.github.io/ccache-and-qmake-qtcreator/
+if [ "$QMAKE_CCACHE" = 1 ]; then
+  qmake_options+=(
+    CONFIG+="ccache"
+  )
+fi
+
 if [ $BUILD_EXPERT = 1 ]; then
   qmake_options+=(
     QMAKE_AR="$AR cqs"
@@ -571,6 +713,7 @@ if [ $BUILD_EXPERT = 1 ]; then
     QMAKE_CXXFLAGS="$CXXFLAGS"
     QMAKE_CXXFLAGS_RELEASE=
     QMAKE_CXXFLAGS_DEBUG=
+    QMAKE_LIBS="$LIBS"
     QMAKE_LFLAGS="$LDFLAGS"
     QMAKE_LFLAGS_RELEASE=
     QMAKE_LFLAGS_DEBUG=
