@@ -1,5 +1,5 @@
 # KLayout Layout Viewer
-# Copyright (C) 2006-2018 Matthias Koefferlein
+# Copyright (C) 2006-2025 Matthias Koefferlein
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 import pya
 import unittest
+import math
 import sys
 
 class BoxPCell(pya.PCellDeclaration):
@@ -26,6 +27,10 @@ class BoxPCell(pya.PCellDeclaration):
     # provide a descriptive text for the cell
     return "Box(L=" + str(parameters[0]) + ",W=" + ('%.3f' % parameters[1]) + ",H=" + ('%.3f' % parameters[2]) + ")"
   
+  def cell_name(self, parameters):
+    # provide a descriptive text for the cell
+    return "Box_L" + str(parameters[0]).replace("/", "d") + "_W" + ('%.3f' % parameters[1]).replace(".", "p") + "_H" + ('%.3f' % parameters[2]).replace(".", "p")
+  
   def get_parameters(self):
   
     # prepare a set of parameter declarations
@@ -77,61 +82,123 @@ class PCellTestLib(pya.Library):
     sb_cell = self.layout().cell(sb_index)
     sb_cell.shapes(l10).insert(pya.Box(0, 0, 100, 200))
     
-    # register us with the name "MyLib"
+    # register us with the name "PCellTestLib"
     self.register("PCellTestLib")
 
 
-# A PCell based on the declaration helper
+if "PCellDeclarationHelper" in pya.__dict__:
 
-class BoxPCell2(pya.PCellDeclarationHelper):
+  # A PCell based on the declaration helper
 
-  def __init__(self):
+  class BoxPCell2(pya.PCellDeclarationHelper):
 
-    super(BoxPCell2, self).__init__()
-  
-    self.param("layer", self.TypeLayer, "Layer", default = pya.LayerInfo(0, 0))
-    self.param("width", self.TypeDouble, "Width", default = 1.0)
-    self.param("height", self.TypeDouble, "Height", default = 1.0)
+    def __init__(self):
+
+      super(BoxPCell2, self).__init__()
     
-  def display_text_impl(self):
-    # provide a descriptive text for the cell
-    return "Box2(L=" + str(self.layer) + ",W=" + ('%.3f' % self.width) + ",H=" + ('%.3f' % self.height) + ")"
-  
-  def produce_impl(self):
-  
-    dbu = self.layout.dbu
-
-    # fetch the parameters
-    l = self.layer_layer
-    w = self.width / self.layout.dbu
-    h = self.height / self.layout.dbu
+      self.param("layer", self.TypeLayer, "Layer", default = pya.LayerInfo(0, 0))
+      self.param("width", self.TypeDouble, "Width", default = 1.0)
+      self.param("height", self.TypeDouble, "Height", default = 1.0)
+      
+    def display_text_impl(self):
+      # provide a descriptive text for the cell
+      return "Box2(L=" + str(self.layer) + ",W=" + ('%.3f' % self.width) + ",H=" + ('%.3f' % self.height) + ")"
     
-    # create the shape
-    self.cell.shapes(l).insert(pya.Box(-w / 2, -h / 2, w / 2, h / 2))
+    def cell_name_impl(self):
+      # provide a descriptive text for the cell
+      return "Box2_L" + str(self.layer).replace("/", "d") + "_W" + ('%.3f' % self.width).replace(".", "p") + "_H" + ('%.3f' % self.height).replace(".", "p")
     
-  def can_create_from_shape_impl(self):
-    return self.shape.is_box()
+    def wants_lazy_evaluation(self):
+      return True
 
-  def transformation_from_shape_impl(self):
-    return pya.Trans(self.shape.box.center() - pya.Point())
-
-  def parameters_from_shape_impl(self):
-    self.layer = self.layout.get_info(self.layer)
-    self.width = self.shape.box.width() * self.layout.dbu
-    self.height = self.shape.box.height() * self.layout.dbu
+    def produce_impl(self):
     
-class PCellTestLib2(pya.Library):
+      dbu = self.layout.dbu
 
-  def __init__(self):  
-  
-    # set the description
-    self.description = "PCell test lib2"
+      # fetch the parameters
+      l = self.layer_layer
+      w = self.width / self.layout.dbu
+      h = self.height / self.layout.dbu
+      
+      # create the shape
+      self.cell.shapes(l).insert(pya.Box(-w / 2, -h / 2, w / 2, h / 2))
+      
+    def can_create_from_shape_impl(self):
+      return self.shape.is_box()
+
+    def transformation_from_shape_impl(self):
+      return pya.Trans(self.shape.box.center() - pya.Point())
+
+    def parameters_from_shape_impl(self):
+      self.layer = self.layout.get_info(self.layer)
+      self.width = self.shape.box.width() * self.layout.dbu
+      self.height = self.shape.box.height() * self.layout.dbu
+      
+  class PCellTestLib2(pya.Library):
+
+    def __init__(self):  
     
-    # create the PCell declarations
-    self.layout().register_pcell("Box2", BoxPCell2())
+      # set the description
+      self.description = "PCell test lib2"
+      
+      # create the PCell declarations
+      self.layout().register_pcell("Box2", BoxPCell2())
+
+      # register us with the name "PCellTestLib2"
+      self.register("PCellTestLib2")
+
+  # A recursive PCell
+
+  class RecursivePCell(pya.PCellDeclarationHelper):
+
+    def __init__(self):
+
+      super(RecursivePCell, self).__init__()
+    
+      self.param("layer", self.TypeLayer, "Layer", default = pya.LayerInfo(0, 0))
+      self.param("line", self.TypeShape, "Line", default = pya.Edge(0, 0, 10000, 0))
+      self.param("level", self.TypeInt, "Level", default = 1)
+      
+    def display_text_impl(self):
+      # provide a descriptive text for the cell
+      return "RecursivePCell(L=" + str(self.layer) + ",E=" + str(pya.CplxTrans(self.layout.dbu) * self.line) + ",LVL=" + str(self.level)
+    
+    def produce_impl(self):
+    
+      # fetch the parameters
+      l = self.layer_layer
+      e = self.line
+
+      if self.level <= 0:
+        self.cell.shapes(l).insert(e)
+        return
+
+      d3 = e.d() * (1.0 / 3.0)
+      d3n = pya.Vector(-d3.y, d3.x)
+
+      e1 = pya.Edge(e.p1, e.p1 + d3)
+      e2 = pya.Edge(e1.p2, e1.p2 + d3 * 0.5 + d3n * math.cos(math.pi / 6))
+      e3 = pya.Edge(e2.p2, e.p1 + d3 * 2.0)
+      e4 = pya.Edge(e3.p2, e.p2)
+
+      for e in [ e1, e2, e3, e4 ]:
+        t = pya.Trans(e.p1 - pya.Point())
+        cc = self.layout.create_cell("RecursivePCell", { "layer": self.layer, "line": t.inverted() * e, "level": self.level - 1 })
+        self.cell.insert(pya.CellInstArray(cc, t))
+      
+  class PCellTestLib3(pya.Library):
+
+    def __init__(self):  
+    
+      # set the description
+      self.description = "PCell test lib3"
+      
+      # create the PCell declarations
+      self.layout().register_pcell("RecursivePCell", RecursivePCell())
+
+      # register us with the name "PCellTestLib3"
+      self.register("PCellTestLib3")
 
-    # register us with the name "MyLib"
-    self.register("PCellTestLib2")
 
 
 def inspect_LayerInfo(self):
@@ -158,6 +225,12 @@ def nh(h):
 
 class DBPCellTests(unittest.TestCase):
 
+  def test_0(self):
+
+    # PCellDeclarationHelper is inside "pya.__all__"
+    if hasattr(pya, "__all__"):
+      self.assertEqual("PCellDeclarationHelper" in pya.__all__, True)
+
   def test_1(self):
 
     # instantiate and register the library
@@ -195,6 +268,9 @@ class DBPCellTests(unittest.TestCase):
     self.assertEqual(pcell_var.is_pcell_variant(), True)
     self.assertEqual(pcell_var.display_title(), "PCellTestLib.Box(L=1/0,W=1.000,H=1.000)")
     self.assertEqual(pcell_var.basic_name(), "Box")
+    self.assertEqual(pcell_var.qname(), "PCellTestLib.Box")
+    self.assertEqual(pcell_var.name, "Box_L1d0_W1p000_H1p000")
+    self.assertEqual(pcell_var.pcell_declaration().wants_lazy_evaluation(), False)
     self.assertEqual(c1.is_pcell_variant(), False)
     self.assertEqual(c1.is_pcell_variant(pcell_inst), True)
     self.assertEqual(pcell_var.pcell_id(), pcell_decl_id)
@@ -304,6 +380,9 @@ class DBPCellTests(unittest.TestCase):
 
   def test_1a(self):
 
+    if not "PCellDeclarationHelper" in pya.__dict__:
+      return
+
     # instantiate and register the library
     tl = PCellTestLib2()
 
@@ -325,6 +404,8 @@ class DBPCellTests(unittest.TestCase):
     pcell_var = ly.cell(pcell_var_id)
     pcell_inst = c1.insert(pya.CellInstArray(pcell_var_id, pya.Trans()))
     self.assertEqual(pcell_var.basic_name(), "Box2")
+    self.assertEqual(pcell_var.name, "Box2_L1d0_W1p000_H1p000")
+    self.assertEqual(pcell_var.qname(), "PCellTestLib2.Box2")
     self.assertEqual(pcell_var.pcell_parameters().__repr__(), "[<1/0>, 1.0, 1.0]")
     self.assertEqual(pcell_var.display_title(), "PCellTestLib2.Box2(L=1/0,W=1.000,H=1.000)")
     self.assertEqual(nh(pcell_var.pcell_parameters_by_name()), "{'height': 1.0, 'layer': <1/0>, 'width': 1.0}")
@@ -338,6 +419,7 @@ class DBPCellTests(unittest.TestCase):
     self.assertEqual(pcell_var.pcell_declaration().__repr__(), pcell_decl.__repr__())
     self.assertEqual(c1.pcell_declaration(pcell_inst).__repr__(), pcell_decl.__repr__())
     self.assertEqual(pcell_inst.pcell_declaration().__repr__(), pcell_decl.__repr__())
+    self.assertEqual(pcell_decl.wants_lazy_evaluation(), True)
 
     li1 = find_layer(ly, "1/0")
     self.assertEqual(li1 == None, False)
@@ -485,6 +567,26 @@ class DBPCellTests(unittest.TestCase):
 
     self.assertEqual(cell.begin_shapes_rec(ly.layer(5, 0)).shape().__str__(), "box (-100,-300;100,300)")
 
+  def test_9(self):
+
+    if not "PCellDeclarationHelper" in pya.__dict__:
+      return
+
+    # instantiate and register the library
+    tl = PCellTestLib3()
+
+    ly = pya.Layout(True)
+
+    li1 = find_layer(ly, "1/0")
+    self.assertEqual(li1 == None, True)
+
+    c1 = ly.create_cell("c1")
+
+    c2 = ly.create_cell("RecursivePCell", "PCellTestLib3", { "layer": pya.LayerInfo(1, 0), "level": 4, "line": pya.Edge(0, 0, 20000, 0) })
+    c1.insert(pya.CellInstArray(c2.cell_index(), pya.Trans()))
+
+    self.assertEqual(c2.display_title(), "PCellTestLib3.RecursivePCell(L=1/0,E=(0,0;20,0),LVL=4")
+    self.assertEqual(str(c1.dbbox()), "(0,0;20,5.774)")
 
 # run unit tests
 if __name__ == '__main__':
